/**
 * 
 */
package org.openrdf.jena;

import info.aduna.collections.iterators.CloseableIterator;
import info.aduna.collections.iterators.IteratorWrapper;

import java.util.Iterator;
import java.util.Vector;

import org.openrdf.model.Resource;
import org.openrdf.model.impl.URIImpl;
import org.openrdf.repository.Repository;
import org.openrdf.repository.RepositoryException;
import org.openrdf.repository.RepositoryResult;
import org.openrdf.sail.SailInitializationException;
import info.aduna.collections.iterators.CloseableIterator;


import com.hp.hpl.jena.graph.Graph;
import com.hp.hpl.jena.graph.Node;
import com.hp.hpl.jena.graph.Triple;
import com.hp.hpl.jena.query.DataSource;
import com.hp.hpl.jena.query.LabelExistsException;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.shared.Lock;

/**
 * This is a wrapper to make a Sesame repository appear as a jena Datasource
 * 
 * @author grimnes
 * 
 */
public class DataSourceSesame extends GraphSesame implements DataSource {

	/**
	 * Default model - may be null - according to Javadoc
	 */
	Model defaultModel = null;

	/**
	 * 
	 * @param rep -
	 *            the repository to wrap
	 * @throws SailInitializationException 
	 */
	public DataSourceSesame(Repository rep) throws SailInitializationException {
		super(rep);		
	}

	/**
	 * @param model
	 *            the default model to set
	 * @see com.hp.hpl.jena.query.DataSource#setDefaultModel(com.hp.hpl.jena.rdf.model.Model)
	 */
	public void setDefaultModel(Model model) {
		defaultModel = model;
	}

	/**
	 * 
	 * 
	 * @param name
	 * @param model
	 * @see com.hp.hpl.jena.query.DataSource#addNamedModel(java.lang.String,
	 *      com.hp.hpl.jena.rdf.model.Model)
	 */
	public void addNamedModel(String name, Model model)
			throws LabelExistsException {
		try {
		if (repository.getConnection().getStatements(null, null, null, false, getContextID(name))
				.hasNext()) {
			throw new LabelExistsException("A model with ID '" + name
					+ "' already exists.");
		}
		Graph g = model.getGraph();
		Resource contextID = getContextID(name);
		
			for (Iterator i = g.find(Node.ANY, Node.ANY, Node.ANY); i.hasNext();) {
				repository.getConnection().add(jena2sesame((Triple) i.next()), contextID);
			}
		} catch (RepositoryException e) {
			throw new RuntimeException(e);
		}

	}

	/**
	 * @param name
	 *            the name of the model to remove
	 * @see com.hp.hpl.jena.query.DataSource#removeNamedModel(java.lang.String)
	 */
	public void removeNamedModel(String name) {
		try {
			repository.getConnection().clear(getContextID(name));
		} catch (RepositoryException e) {
			throw new RuntimeException(e);
		}

	}

	private Resource getContextID(String name) {
		return new URIImpl(name);
	}

	/**
	 * @param name -
	 *            the name to replace
	 * @param model -
	 *            the model to replace with
	 * @see com.hp.hpl.jena.query.DataSource#replaceNamedModel(java.lang.String,
	 *      com.hp.hpl.jena.rdf.model.Model)
	 */
	public void replaceNamedModel(String name, Model model) {
		// TODO: Clever diff?
		try {
			repository.getConnection().setAutoCommit(false);
			removeNamedModel(name);
			addNamedModel(name, model);
			repository.getConnection().setAutoCommit(true);
		} catch (RepositoryException e) {
			// TODO: Is this right?
			try {
				repository.getConnection().rollback();
			} catch (RepositoryException e2) {
				throw new RuntimeException(
						"Could not replace model, and could not rollback!", e2);
			}
			throw new RuntimeException("Could not replace model:", e);
		}
	}

	/**
	 * @see com.hp.hpl.jena.query.Dataset#getDefaultModel()
	 */
	public Model getDefaultModel() {
		return defaultModel;
	}

	/**
	 * @see com.hp.hpl.jena.query.Dataset#getNamedModel(java.lang.String)
	 */
	public Model getNamedModel(String name) {
		
		try {
			return new ModelSesame(new NamedGraphSesame(repository,
					getContextID(name)));
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.hp.hpl.jena.query.Dataset#containsNamedModel(java.lang.String)
	 */
	public boolean containsNamedModel(String arg0) {
		return false;
	}

	/**
	 * 
	 * 
	 * @see com.hp.hpl.jena.query.Dataset#listNames()
	 */
	public Iterator listNames() {
		try {
		//Does this really want strings?
		Vector<String> names=new Vector<String>(); 
		//we need to step through the iterator here to make sure it's closed.
		RepositoryResult<Resource> contextIds = repository.getConnection().getContextIDs();
		Iterator i = contextIds.asCollection().iterator();
		while(i.hasNext()) names.add(i.next().toString());
		contextIds.close();
		return names.iterator();
		} catch (RepositoryException e) {
			throw new RuntimeException(e);
		}
	}

	/**
	 * Jena has locks?
	 * 
	 * @see com.hp.hpl.jena.query.Dataset#getLock()
	 */
	public Lock getLock() {
		//return new LockSesame(repository);
		throw new RuntimeException("Lock are not implemented yet, in DataSourceSesame.java .");
	}

}
