package org.openrdf.jena;

import java.util.List;

import org.openrdf.jena.GraphSesame;
import org.openrdf.repository.Repository;


import com.hp.hpl.jena.graph.BulkUpdateHandler;
import com.hp.hpl.jena.graph.Graph;
import com.hp.hpl.jena.graph.Triple;
import com.hp.hpl.jena.reasoner.Reasoner;
import com.hp.hpl.jena.reasoner.rulesys.BasicForwardRuleInfGraph;

/**
 * 
 * This class wraps a sesame repository as a jena InfGraph
 * It handles bulkupdates efficiently in single Sesame transactions.
 * IT ONLY HANDLES FORWARD CHAINING REASONERS (cause this is all we need :) 
 * 
 * @author grimnes
 *
 */
public class InfGraphSesame extends BasicForwardRuleInfGraph {

	Repository repository;
	GraphSesame sesamegraph;

	public InfGraphSesame(Reasoner reasoner, List rules, Graph schema) {
		super(reasoner, rules, schema);
	}

	public InfGraphSesame(Reasoner reasoner, Graph schema) {
		super(reasoner, schema);
	}
	
	public InfGraphSesame(Reasoner reasoner, List rules, Graph schema, Graph data) {
		super(reasoner, rules, schema, data);
		try {
			repository=((GraphSesame)data).getRepository();
			sesamegraph=(GraphSesame)data;
		} catch (ClassCastException e) {
			
		}
	}
	
	protected void hackThisIntoEngine(Triple trip) {
        if (isPrepared) {
            engine.add(trip);
        }
	}

	@Override
	public BulkUpdateHandler getBulkUpdateHandler() {
		if (bulkHandler == null) bulkHandler = new SesameInfBulkUpdateHandler( this );
        return bulkHandler;
	}

	@Override
	public void rebind(Graph data) {
		try {
			repository=((GraphSesame)data).getRepository();
			sesamegraph=(GraphSesame)data;
		} catch (ClassCastException e) {
			
		}
		super.rebind(data);
	}

	

}
