/*
 * Created on 16-Jun-2005
 * 
 */
package org.openrdf.jena;

import org.openrdf.model.Resource;
import org.openrdf.repository.Repository;
import org.openrdf.repository.RepositoryException;



import com.hp.hpl.jena.graph.Graph;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.impl.ModelCom;

/**
 * Jena-Sesame-Model: A Jena model using Sesame as a backend.
 * @author wf
 * 
 */
public class ModelSesame extends ModelCom {

	/**
	 * @param base
	 */
	public ModelSesame(Graph base) {
		super(base);
	}
	
	/**
	 * Create a default Jena-Sesame-Model, which uses the given Sesame repository.
	 * @param rep
	 */
	public ModelSesame(Repository rep) {
		this(new GraphSesame(rep));
	}
	
	/***
	 * create a new named graph model, pass in the context to use for read and write.
	 * Context ist an openrdf uri
	 */
	public ModelSesame(Repository rep, Resource context)
	{
		this(new NamedGraphSesame(rep, context));
	}
    
    /**
     * Create a default Jena-Sesame-Model, which uses a Sesame in-memory local repository, 
     * and does not have persistent support, synchroinzation support and inference capability.
     *  
     */
	public static ModelSesame createDefaultModel() {
    	return new ModelSesame(new GraphSesame());
    }

    /**
     * Create a customized Jena-Sesame-Model, which uses a Sesame in-memory local repository,
     * @param inferencing   true: the model has inference capability; false: not.   
     * @param sync          true: the model is synchronized; false: not.
     * @param fileName      the name of the file that acts as the persistent storage.
     *                      Null means no persistent support required.
     * @param format        the RDF data format in the persistent storage. It can be
     *                      rdfxml, ntriples, turtle, null (use the default which is ntriples).
     * @return              a Jena-Sesame-Model
     */
    public static ModelSesame createMemoryModel(boolean inferencing, boolean sync, 
            String fileName, String format) {
    	return new ModelSesame(new GraphSesame(inferencing, sync, fileName, format));
    }

	/**
     * Create a Jena-Sesame-Model, which uses a Sesame native local repository,
     * @param sync  true: the model is synchronized; false: not.
     * @param dir   specifies the directory that can be used by the native sail to store its files.
	 * @return      a Jena-Sesame-Model
	 */
	public static ModelSesame createNativeModel(boolean sync, String dir) {
		return new ModelSesame(new GraphSesame(sync, dir));
	}

	/**
     * Create a Jena-Sesame-Model, which uses a Sesame rdbms local repository,
	 * @param sync     true: the model is synchronized; false: not.
	 * @param driver   identifies the JDBC (Java Data Base Connectivity) driver that is to be used to access the database.
	 * @param url      identifies the location of the database through a URL.
	 * @param user     identifies a username with which Sesame can access the database. 
	 * @param password identifies a password with which Sesame can access the database.
	 * @return         a Jena-Sesame-Model
	 */
	public static Model createDatabaseModel(boolean sync, String driver, String url, String user, String password) {
		return new ModelSesame(new GraphSesame(sync, driver, url, user, password));
	}

	@Override
	public Model removeAll() {
		try {
			getRepository().getConnection().clear();
		} catch (ClassCastException e) {
			super.removeAll();
		} catch (RepositoryException e) {
			super.removeAll();
		}
		return this;
	}
	
	public Repository getRepository() {
		return ((GraphSesame)getGraph()).getRepository();
	}	
}