/**
 * 
 */
package org.openrdf.jena;

import java.util.Iterator;

import org.openrdf.model.Resource;
import org.openrdf.model.Statement;
import org.openrdf.repository.Repository;
import org.openrdf.repository.RepositoryException;
import org.openrdf.repository.RepositoryResult;


import info.aduna.collections.iterators.CloseableIterator;
import info.aduna.collections.iterators.IteratorWrapper;

import com.hp.hpl.jena.graph.BulkUpdateHandler;
import com.hp.hpl.jena.graph.Triple;
import com.hp.hpl.jena.graph.TripleMatch;
import com.hp.hpl.jena.util.iterator.ExtendedIterator;

/**
 * This pretends to be a named graph. You can set and get the context-id to which
 * this graph reads and writes. Also the BulkUpdateHandler works with it.
 * <b>UseContextOnRead</b>:This can be used
	 * to tweak an inferencer or query engine: the inference should work on all
	 * triples, but changes only be made to a specific model
 * TODO: re-ification i haven't really considered at all -= with some luck it will JUST WORK (tm)
 * @author grimnes
 */
public class NamedGraphSesame extends GraphSesame {

	/**
	 * the context id from which data will be read or written to.
	 */
	Resource contextID;
	
	/**
	 * if true, the context will be used to filter reading operations.
	 * If false, reading this graph will return all triples. This can be used
	 * to tweak an inferencer or query engine: the inference should work on all
	 * triples, but changes only be made to a specific model.
	 * Default is true.
	 */
	boolean useContextOnRead = true;
	
	public NamedGraphSesame(Repository repository,Resource contextID)  {
		super(repository);
		this.contextID=contextID;
	}
	
	/**
	 * Create a NamedGraphSesame
	 * @param repository the base repository
	 * @param contextID the contextid to use to filter reads and writes
	 * @param useContextOnRead 	if true, the context will be used to filter reading operations.
	 * If false, reading this graph will return all triples. This can be used
	 * to tweak an inferencer or query engine: the inference should work on all
	 * triples, but changes only be made to a specific model.
	 * Default is true.
	 */
	public NamedGraphSesame(Repository repository,Resource contextID, boolean useContextOnRead) {
		super(repository);
		this.contextID=contextID;
	}
	
	/** 
	 * We must override this to take the context into account.
	 * TODO: hopefully we can push this into sesame later. 
	 */
	protected int graphBaseSize() {
		try {
		RepositoryResult<Statement> statements = repository.getConnection().getStatements(null,null,null,false,contextID);
        int tripleCount = 0;        
        Iterator<Statement> it = statements.asCollection().iterator();        
        while (it.hasNext()) { it.next(); tripleCount += 1; }
        //statements.close();
        return tripleCount;
		} catch (RepositoryException e) {
			throw new RuntimeException(e);
		}
		
	}

	/**
	 * This must be overridden to pass the right contextID
	 * 
	 * @see com.hp.hpl.jena.graph.impl.GraphBase#graphBaseFind(com.hp.hpl.jena.graph.TripleMatch)
	 */
	protected ExtendedIterator graphBaseFind(TripleMatch tm) {
		if (!useContextOnRead)
			return super.graphBaseFind(tm);
		
		// throw new RuntimeException("GraphSesame::graphBaseFind");
		Statement stri= jena2sesame(tm.asTriple());
		//logger.debug("\tJena: " + tm + "\n\tSesame: " + stri);
		
		try {
		RepositoryResult<Statement> stit = repository.getConnection().getStatements(
				stri.getSubject(), stri.getPredicate(), stri.getObject(), false, contextID);
		return new ExtendedStatementIterator(new IteratorWrapper<Statement>(stit.asCollection().iterator()));
		} catch (RepositoryException e) {
			throw new RuntimeException(e);
		}
		
		
	}
	
	/** 
	 * @see com.hp.hpl.jena.graph.impl.GraphWithPerform#performAdd(com.hp.hpl.jena.graph.Triple)
	 */
	public void performAdd(Triple t) {
		Statement stri= jena2sesame(t);
//		logger.debug("\tJena: " + t + "\n\tSesame: " + stri);
		try {
			repository.getConnection().add(stri.getSubject(), stri.getPredicate(), stri.getObject(),contextID);
		} catch (RepositoryException e) {
			throw new RuntimeException(e);
		}
	}
	
	/**
	 * @see com.hp.hpl.jena.graph.impl.GraphWithPerform#performDelete(com.hp.hpl.jena.graph.Triple)
	 */
	public void performDelete(Triple t) {
		Statement stri= jena2sesame(t);
//		logger.debug("\tJena: " + t + "\n\tSesame: " + stri);
		try {
			repository.getConnection().remove(stri.getSubject(), stri.getPredicate(), stri.getObject(),contextID);
		} catch (RepositoryException e) { 			
			throw new RuntimeException(e);
		}		   
	}

	/**
	 * the context id from which data will be read or written to.
	 */
	public Resource getContextID() {
		return contextID;
	}

	/**
	 * the context id from which data will be read or written to.
	 */
	public void setContextID(Resource contextID) {
		this.contextID = contextID;
	}

	@Override
	public BulkUpdateHandler getBulkUpdateHandler() {
		return new NamedSesameBulkUpdateHandler(this);
	}

	
	/**
	 * if true, the context will be used to filter reading operations.
	 * If false, reading this graph will return all triples. This can be used
	 * to tweak an inferencer or query engine: the inference should work on all
	 * triples, but changes only be made to a specific model.
	 * Default is true.
	 */
	public boolean isUseContextOnRead() {
		return useContextOnRead;
	}
	
	
	
}
