/**
 * 
 */
package org.openrdf.jena;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.gnowsis.util.GnowsisError;
import org.openrdf.model.Resource;
import org.openrdf.model.Statement;
import org.openrdf.model.URI;
import org.openrdf.repository.RepositoryConnection;
import org.openrdf.repository.RepositoryException;
import org.openrdf.repository.RepositoryResult;
//import org.openrdf.sesame.repository.Transaction;
import info.aduna.collections.iterators.CloseableIterator;

import com.hp.hpl.jena.graph.Graph;
import com.hp.hpl.jena.graph.GraphEvents;
import com.hp.hpl.jena.graph.GraphUtil;
import com.hp.hpl.jena.graph.Node;
import com.hp.hpl.jena.graph.Triple;
import com.hp.hpl.jena.graph.impl.SimpleBulkUpdateHandler;
import com.hp.hpl.jena.util.iterator.ExtendedIterator;

/**
 *  A bulk update handler that works with context.
 * @author sauermann
 */
public class NamedSesameBulkUpdateHandler extends SimpleBulkUpdateHandler {

	/**
	 * shortcut to the named graph
	 */
	NamedGraphSesame parent;
	
	public NamedSesameBulkUpdateHandler(NamedGraphSesame parent) {
		super(parent);
		this.parent = parent;
	}

	/**
	 * This might be slow, but not as slow as doing one transaction per statement.
	 * @see com.hp.hpl.jena.graph.impl.SimpleBulkUpdateHandler#add(com.hp.hpl.jena.graph.Graph)
	 */
	@Override
	public void add(Graph g) {
		GraphSesame sesamegraph=(GraphSesame)graph;
		try {
			Resource context = parent.contextID;
			RepositoryConnection c = sesamegraph.repository.getConnection();
			c.setAutoCommit(false);
			//Transaction t=sesamegraph.repository.startTransaction();
			for (ExtendedIterator ei=g.find(Node.ANY,Node.ANY,Node.ANY);ei.hasNext();) {
				c.add(sesamegraph.jena2sesame((Triple)ei.next()), context);
			}
			c.commit();
			c.close();
		} catch (RepositoryException e) {
			throw new GnowsisError("Couldn't create transaction",e);
		}
	}
	
	@Override
	protected void add(List triples, boolean notify) {
		GraphSesame sesamegraph=(GraphSesame)graph;
		try {
			Resource context = parent.contextID;
			RepositoryConnection c = sesamegraph.repository.getConnection();
			c.setAutoCommit(false);
			//Transaction t=sesamegraph.repository.startTransaction();
			for (Iterator i = triples.iterator(); i.hasNext(); ) {
				Triple trip = (Triple) i.next();
				c.add(sesamegraph.jena2sesame(trip), context);
			}
			c.commit();
			c.close();			
		} catch (RepositoryException e) {
			throw new GnowsisError("Couldn't create transaction",e);
		}
		if (notify) manager.notifyAddList( graph, triples );
	}

	@Override
	public void remove(Node s, Node p, Node o) {
		GraphSesame sesamegraph=(GraphSesame)graph;
		try {
			Resource context = parent.contextID;
			// select them
			RepositoryResult<Statement> read;
			if (parent.isUseContextOnRead())
				read = sesamegraph.repository.getConnection().getStatements(
						(Resource) ValueToNode.reverse(s), 
						(URI)ValueToNode.reverse(p), 
						ValueToNode.reverse(o), 
						false, context);
			else
				read = sesamegraph.repository.getConnection().getStatements(
						(Resource) ValueToNode.reverse(s), 
						(URI)ValueToNode.reverse(p), 
						 ValueToNode.reverse(o), false  );
			RepositoryConnection c = sesamegraph.repository.getConnection();
			c.setAutoCommit(false);
			//Transaction t=sesamegraph.repository.startTransaction();
			c.remove(read, context);
			c.commit();
			c.close();
			
		} catch (RepositoryException e) {
			throw new GnowsisError("Couldn't create transaction: "+e,e);
		}
		manager.notifyEvent( graph, GraphEvents.remove( s, p, o ) );
	}

}
