package org.openrdf.jena;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.gnowsis.util.GnowsisError;
import org.gnowsis.util.GnowsisException;
import org.openrdf.model.Value;
import org.openrdf.query.GraphQuery;
import org.openrdf.query.MalformedQueryException;
import org.openrdf.query.QueryHandler;
import org.openrdf.query.QueryLanguage;
import org.openrdf.query.TupleQuery;
import org.openrdf.query.MalformedQueryException;
import org.openrdf.query.QueryLanguage;
import org.openrdf.query.TupleQuery;
import org.openrdf.repository.Repository;
import org.openrdf.repository.sail.SailRepository;

import info.aduna.collections.iterators.CloseableIterator;

import com.hp.hpl.jena.graph.Node;
import com.hp.hpl.jena.graph.Triple;
import com.hp.hpl.jena.graph.query.QueryHandler;

import com.hp.hpl.jena.query.QueryExecution;
import com.hp.hpl.jena.query.QuerySolution;
import com.hp.hpl.jena.query.QuerySolutionMap;
import com.hp.hpl.jena.query.ResultSet;
import com.hp.hpl.jena.query.core.ElementBlock;
import com.hp.hpl.jena.query.util.Context;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.rdf.model.RDFNode;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.util.FileManager;


/**
 * Run ARQ SPARQL queries on a Sesame Repository
 * @author Daniel Burkhart
 * @author grimnes
 */
public class QueryExecutionSesame implements QueryExecution {

	Model model = ModelFactory.createDefaultModel();

	private Logger logger = Logger.getLogger(QueryExecutionSesame.class.getName());

	protected String query = null;

	protected Repository rep = null;

	static int queryCount = 0;

	boolean queryInitialised = false;

	private QueryLanguage querylanguage;

	public QueryExecutionSesame(String q, Repository r, QueryLanguage ql) {
		query = q;
		rep = r;
		querylanguage = ql;
	}

	public void setFileManager(FileManager arg0) {

	}

	public void setInitialBinding(QuerySolution arg0) {

	}

	public Context getContext() {
		return null;
	}

	public ResultSet execSelect() {
		try {
			if (rep != null) {
				TupleQuery sesameQuery=QueryHandler.parseTupleQuery(querylanguage,query);
				
				//iter = (RValueTupleIterator) rep.evaluateTupleQuery(querylanguage, query);
				CloseableIterator<List<Value>> iter = sesameQuery.evaluate(((SailRepository)rep).getSail());
				return createResultSet(sesameQuery,iter);
			} else {
				logger.severe("Repository not initialized");
				return null;
			}

		} catch (Exception e) {
			// Right - we have to follow Jena policy - so no exceptions
			throw new GnowsisError("unwrap me",e);
		}
	}

	/**
	 * creates a ResultSet for a query q and RValueTupleIterator
	 * 
	 * @param sesameQuery
	 * @param iter
	 * @return a resultset for the query q
	 * @throws MalformedQueryException 
	 */
	private ResultSet createResultSet(TupleQuery sesameQuery, CloseableIterator<List<Value>> iter) throws MalformedQueryException {			
		List<String> variables=Arrays.asList(sesameQuery.getQueryModel().getColumnHeaders());
		
		//ResultSetImpl result = new ResultSetImpl(createResultsVars(q));
		ResultSetImpl result=new ResultSetImpl(variables);

		while (iter.hasNext()) {
			QuerySolutionMap qs = new QuerySolutionMap();
			ArrayList l = (ArrayList) iter.next();
			for (int i = 0; i < l.size(); i++) {

				// create either a resource or a literal
				Value o = (Value) l.get(i);
				// hm ,this log message throws two null pointer exceptions at once, 
				// when doing optional queries
				//logger.finest(o.getClass().getName() + ":" + o.toString());
				Node node = ValueToNode.convert(o);
				//RDFNode rdfNode = model.getRDFNode(node);
				RDFNode rdfnode=null;
				if (node != null)
					rdfnode=model.asRDFNode(node);
				qs.add(result.getResultVars().get(i).toString(), rdfnode);
			}
			result.add(qs);
		}
		iter.close();
		return result;
	}

	/**
	 * reads out the variable Names from a query. If a "*" was used after
	 * SELECT/DESCRIBE/CONSTRUCT, the query has to be parsed
	 * 
	 * @param q
	 * @return a list of variable names
	 */
	private List<String> createResultsVars(Query q) {
		if (q.isQueryResultStar()) {
			List<String> l = new ArrayList<String>();
			ElementBlock el = q.getQueryBlock();
			Set s = el.varsMentioned();
			for (Iterator i = s.iterator(); i.hasNext();) {
				l.add((String) i.next());
			}
			logger.finer("variables found in query: " + l.getClass().getName()
					+ " : " + l.toString());
			return l;
		} else
			return q.getResultVars();
	}

	public Model execConstruct() {
		Model result = ModelFactory.createDefaultModel();
		try {
			if (rep != null) {
				if (querylanguage.equals(QueryLanguage.SPARQL))
					throw new GnowsisError(
							"SPARQL is not supported yet for CONSTRUCT queries. Use SERQL instead");

				//run the CONSTRUCT query on the Sesame repository and map the resulting
				//Iterator to a Jena Model
				 
					Query q = rep.getConnection().prepareQuery(
						querylanguage, query);
					CloseableIterator<RStatement> iter =
				ExtendedStatementIterator extIter = new ExtendedStatementIterator(
						iter);
				while (extIter.hasNext()) {
					Resource s;
					Resource p;
					RDFNode o;
					Triple t = (Triple) extIter.next();
					s = (Resource) result.getRDFNode(t.getSubject());
					p = (Resource) result.getRDFNode(t.getPredicate());
					o = result.getRDFNode(t.getObject());
					result.add(s, model.createProperty(p.getURI()), o);
					
				}
			} else {
				logger.severe("Repository not initialized");
				return null;
			}

		} catch (MalformedQueryException e) {
			//no exceptions allowed, so error:
			throw new GnowsisError("Unwrap me!",e);
		}
		return result;
	}

	public Model execDescribe() {
		return null;
	}

	public boolean execAsk() {
		return false;
	}

	public void abort() {

	}

	public void close() {

	}
}