/**
 * 
 */
package org.openrdf.jena;

import java.util.Iterator;
import java.util.List;

import org.gnowsis.util.GnowsisError;
import org.openrdf.repository.RepositoryConnection;
import org.openrdf.repository.RepositoryException;


import com.hp.hpl.jena.graph.Graph;
import com.hp.hpl.jena.graph.GraphUtil;
import com.hp.hpl.jena.graph.Node;
import com.hp.hpl.jena.graph.Triple;
import com.hp.hpl.jena.graph.impl.SimpleBulkUpdateHandler;
import com.hp.hpl.jena.util.iterator.ExtendedIterator;

/**
 * 
 * This is a bulkhandler that knows about sesame transactions. 
 * 
 *  TODO: Override more methods here as the need arises.
 * 
 * @author grimnes
 *
 */
public class SesameBulkUpdateHandler extends SimpleBulkUpdateHandler {

	public SesameBulkUpdateHandler(GraphSesame parent) {
		super(parent);
	}

	/**
	 * This might be slow, but not as slow as doing one transaction per statement.
	 * @see com.hp.hpl.jena.graph.impl.SimpleBulkUpdateHandler#add(com.hp.hpl.jena.graph.Graph)
	 */
	@Override
	public void add(Graph g) {
		GraphSesame sesamegraph=(GraphSesame)graph;
		try {
			RepositoryConnection c = sesamegraph.repository.getConnection();
			c.setAutoCommit(false);
			//Transaction t=sesamegraph.repository.startTransaction();
			for (ExtendedIterator ei=g.find(Node.ANY,Node.ANY,Node.ANY);ei.hasNext();) {
				c.add(sesamegraph.jena2sesame((Triple)ei.next()));
			}
			c.commit();
			c.close();
		} catch (RepositoryException e) {
			throw new GnowsisError("Couldn't create transaction",e);
		}
	}
	
    
    public void add( Graph g, boolean withReifications )
        { 
        addIterator( GraphUtil.findAll( g ), false );  
        if (withReifications) addReifications( graph, g );
        manager.notifyAddGraph( graph, g );
        }

	@Override
	protected void add(List triples, boolean notify) {
		GraphSesame sesamegraph=(GraphSesame)graph;
		try {
			sesamegraph.repository.getConnection().setAutoCommit(false);
			for (Iterator i = triples.iterator(); i.hasNext(); ) {
				Triple trip = (Triple) i.next();
				sesamegraph.repository.getConnection().add(sesamegraph.jena2sesame(trip));
			}
			sesamegraph.repository.getConnection().commit();
			sesamegraph.repository.getConnection().setAutoCommit(true);
		} catch (RepositoryException e) {
			throw new GnowsisError("Couldn't create transaction",e);
		}
		if (notify) manager.notifyAddList( graph, triples );
	}
	
	
	
	


}
