/*
 * Copyright Aduna (http://www.aduna-software.com/) (c) 2007.
 *
 * Licensed under the Aduna BSD-style license.
 */
package org.openrdf.model.impl;

import java.math.BigDecimal;
import java.math.BigInteger;

import org.openrdf.model.URI;
import org.openrdf.model.vocabulary.XMLSchema;

/**
 * An extension of {@link LiteralImpl} that stores an integer value using a
 * {@link BigDecimal} object.
 * 
 * @author Arjohn Kampman
 */
public class DecimalLiteralImpl extends LiteralImpl {

	private BigDecimal _value;

	/**
	 * Creates an xsd:decimal literal with the specified value.
	 */
	public DecimalLiteralImpl(BigDecimal value) {
		this(value, XMLSchema.DECIMAL);
	}

	/**
	 * Creates a literal with the specified value and datatype.
	 */
	public DecimalLiteralImpl(BigDecimal value, URI datatype) {
		// TODO: maybe DecimalLiteralImpl should not extend LiteralImpl?
		super(value.toPlainString(), datatype);
		_value = value;
	}

	@Override
	public byte byteValue()
	{
		return _value.byteValue();
	}

	@Override
	public short shortValue()
	{
		return _value.shortValue();
	}

	@Override
	public int intValue()
	{
		return _value.intValue();
	}

	@Override
	public long longValue()
	{
		return _value.longValue();
	}

	@Override
	public float floatValue()
	{
		return _value.floatValue();
	}

	@Override
	public double doubleValue()
	{
		return _value.doubleValue();
	}

	@Override
	public BigInteger integerValue()
	{
		return _value.toBigInteger();
	}

	@Override
	public BigDecimal decimalValue()
	{
		return _value;
	}
}
